( cd usr/lib ; rm -rf libpt_linux_x86_r.so.1.8 )
( cd usr/lib ; ln -sf libpt_linux_x86_r.so.1.8.7 libpt_linux_x86_r.so.1.8 )
( cd usr/lib ; rm -rf libpt_linux_x86_r.so.1 )
( cd usr/lib ; ln -sf libpt_linux_x86_r.so.1.8.7 libpt_linux_x86_r.so.1 )
( cd usr/lib ; rm -rf libpt_linux_x86_r.so )
( cd usr/lib ; ln -sf libpt_linux_x86_r.so.1.8.7 libpt_linux_x86_r.so )
( cd usr/lib ; rm -rf libpt.so )
( cd usr/lib ; ln -sf libpt_linux_x86_r.so libpt.so )
( cd usr/bin ; rm -rf ptlib-config )
( cd usr/bin ; ln -sf ../share/pwlib/make/ptlib-config ptlib-config )
